function [CRB] = TDOALocUknPropSpd_CRLB(senPos, srcLoc, c, varargin)
% [CRB] = TDOALocUknPropSpd_CRLB(senPos, srcLoc, c, varargin)
%
% Y. Sun, K. C. Ho, Y. Yang, and L. Chen, "An asymptotically optimal
% estimator for source location and propagation speed by TDOA," IEEE Signal
% Process. Lett., vol. 30, pp. 1037-1041, Aug. 2023.
%
% CRLB for TDOA localization when the propagation speed is unknown. The
% CRLB contains source position and propagation speed, compatible for 2D
% and 3D cases, with or without sensor position errors
%
% Input:
%   senPos(m):	(Dim x M), the postions of reciveing sensors, each row is a sensor position and
%                   first row is the reference sensor;
%   souLoc(m):	(Dim x 1), the postions of reciveing source;
%   c(m/s):     scalar, the propagation speed of signal;
%   Qr:         TDOA measurements covariance matrix;
%   *Qs:        sensor positions covariance matrix;
% Output:
%   CRB:	(Dim+1 x Dim+1) matrix for position and propagation speed.
%
% Created by Y. Sun, K. C. Ho   Oct. 2023
%

Qt = varargin{1};
[N,M] = size(senPos);   % N = Dim, M = num. of sensors 
for i = 1:M-1
    dt_du(i,:) = [(srcLoc-senPos(:,i+1))/(norm(srcLoc-senPos(:,i+1))*c)-(srcLoc-senPos(:,1))/(norm(srcLoc-senPos(:,1))*c);
        -norm(srcLoc-senPos(:,i+1))/(c^2)+norm(srcLoc-senPos(:,1))/(c^2)]';
end

% X = dt_du'/Qt*dt_du;
% if length(varargin) == 1
%     CRB = inv(X);
% elseif length(varargin) == 2
%     Qs = varargin{2};
%     dt_ds = zeros(M-1,N*M);
%     
%     for j = 2 : M    
%         dt_ds((j-1),:) = [(srcLoc-senPos(:,1))'/norm(srcLoc-senPos(:,1)), zeros(1,(j-2)*N), ...
%             -(srcLoc-senPos(:,j))'/norm(srcLoc-senPos(:,j)), zeros(1,(M-j)*N)]/c;
%     end
% 
%     Y = dt_du' * inv(Qt) * dt_ds;
%     Z = inv(Qs) + dt_ds'* inv(Qt) * dt_ds;
%     
%     CRB = inv(X) + inv(X)*Y*inv(Z-Y'*inv(X)*Y)*Y'*inv(X);
% else
%     error('Too many input args.')
% end

% r = sqrt(sum((srcLoc - senPos).^2, 1))';
r = sqrt(sum((repmat(srcLoc,1,size(senPos,2)) - senPos).^2, 1))';
B1 = 2*c*diag(r(2:end));
C1(:,1:N) = 2*kron((srcLoc-senPos(:,1))',ones(M-1,1));
for i = 1:M-1
    C1(i, (1:N)+i*N) = -2*(srcLoc-senPos(:,i+1))';
end

Qs = 0;
if length(varargin) == 2
    Qs = varargin{2};
end
Q = Qt + B1\C1*Qs*C1'/B1;
CRB = inv(dt_du'/Q*dt_du);

